package co.paystack.flutter_theoaks_paystack

import android.app.Activity
import android.provider.Settings
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** FlutterTheoaksPaystackPlugin */
class FlutterTheoaksPaystackPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private lateinit var authDelegate: AuthDelegate
  private lateinit var activity: Activity

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "flutter_theoaks_paystack")
    channel.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
      "getDeviceId" -> {
        val deviceId = Settings.Secure.getString(
          activity.contentResolver,
          Settings.Secure.ANDROID_ID
        )
        result.success("androidsdk_$deviceId")
      }
      "getAuthorization" -> {
        authDelegate.handleAuthorization(result, call)
      }
      "getEncryptedData" -> {
        val encryptedData = Crypto.encrypt(call.argument<String>("stringData").toString())
        result.success(encryptedData)
      }

      else -> result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onDetachedFromActivity() {

  }


  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity
    authDelegate = AuthDelegate(binding.activity)
  }

  override fun onDetachedFromActivityForConfigChanges() = onDetachedFromActivity()

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) = onAttachedToActivity(binding)
}
